#!/usr/bin/sh

python3.11 lsp-specification-3-17.py
python3.10 lsp-specification-3-17.py
python3.9 lsp-specification-3-17.py
python3.8 lsp-specification-3-17.py
python3.7 lsp-specification-3-17.py
python3.6 lsp-specification-3-17.py
pypy3 lsp-specification-3-17.py
